@@ -1,7 +1,7 @@
 package Perinci::Sub::Wrapper;
 
-our $DATE = '2015-09-04'; # DATE
-our $VERSION = '0.79'; # VERSION
+our $DATE = '2016-01-20'; # DATE
+our $VERSION = '0.80'; # VERSION
 
 use 5.010001;
 use strict;
@@ -686,7 +686,7 @@ sub _handle_args {
                 );
                 if ($argspec->{stream}) {
                     $self->push_lines(
-                        'if ('."\$err_$dn".') { die "Record #$i of streaming argument '."'$prefix$argname'".' fails validation: '."\$err_$dn".'" }',
+                        'if ('."\$err_$dn".') { die "Record #$i of streaming argument '."'$prefix$argname'".' ($rec) fails validation: '."\$err_$dn".'" }',
                         '$rec;',
                     );
                 } else {
@@ -922,7 +922,7 @@ sub handle_result {
             $self->push_lines("$cd->{result};");
             $self->_errif(
                 500,
-                qq["BUG: Result from sub $sub_name fails validation: ].
+                qq["BUG: Result from sub $sub_name (\$_w_res2) fails validation: ].
                     qq[\$_w_err2_res"],
                 "\$_w_err2_res");
             $self->unindent;
@@ -1476,7 +1476,7 @@ Perinci::Sub::Wrapper - A multi-purpose subroutine wrapping framework
 
 =head1 VERSION
 
-This document describes version 0.79 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2015-09-04.
+This document describes version 0.80 of Perinci::Sub::Wrapper (from Perl distribution Perinci-Sub-Wrapper), released on 2016-01-20.
 
 =head1 SYNOPSIS
 
@@ -1558,6 +1558,74 @@ wrapping function dynamically (during runtime), or
 L<Dist::Zilla::Plugin::Rinci::Wrap> on an example of embedding the generated
 wrapping code to source code during build.
 
+=head1 EXTENDING
+
+The framework is simple and extensible. Please delve directly into the source
+code for now. Some notes:
+
+The internal uses OO.
+
+The main wrapper building mechanism is in the C<wrap()> method.
+
+For each Rinci property, it will call C<handle_NAME()> wrapper handler method.
+The C<handlemeta_NAME()> methods are called first, to determine order of
+processing. You can supply these methods either by subclassing the class or,
+more simply, monkeypatching the method in the C<Perinci::Sub::Wrapper> package.
+
+The wrapper handler method will be called with a hash argument, containing these
+keys: B<value> (property value), B<new> (this key will exist if C<convert>
+argument of C<wrap()> exists, to convert a property to a new value).
+
+For properties that have name in the form of C<NAME1.NAME2.NAME3> (i.e., dotted)
+only the first part of the name will be used (i.e., C<handle_NAME1()>).
+
+=head1 VARIABLES
+
+=head2 $Log_Wrapper_Code (BOOL)
+
+Whether to log wrapper result. Default is from environment variable
+LOG_PERINCI_WRAPPER_CODE, or false. Logging is done with L<Log::Any> at trace
+level.
+
+=head1 RINCI FUNCTION METADATA
+
+=head2 x.perinci.sub.wrapper.disable_validate_args => bool
+
+Can be set to 1 to set C<validate_args> to 0 by default. This is used e.g. if
+you already embed/insert code to validate arguments by other means and do not
+want to repeat validating arguments. E.g. used if you use
+L<Dist::Zilla::Plugin::Rinci::Validate>.
+
+=head2 x.perinci.sub.wrapper.disable_validate_result => bool
+
+Can be set to 1 to set C<validate_result> to 0 by default. This is used e.g. if
+you already embed/insert code to validate result by other means and do not want
+to repeat validating result. E.g. used if you use
+L<Dist::Zilla::Plugin::Rinci::Validate>.
+
+=head2 x.perinci.sub.wrapper.logs => array
+
+Generated/added by this module to the function metadata for every wrapping done.
+Used to avoid adding repeated code, e.g. to validate result or arguments.
+
+=head1 PERFORMANCE NOTES
+
+The following numbers are produced on an Intel Core i5-2400 3.1GHz desktop using
+PSW v0.51 and Perl v5.18.2. Operating system is Debian sid (64bit).
+
+For perspective, empty subroutine (C<< sub {} >>) as well as C<< sub { [200,
+"OK"] } >> can be called around 5.3 mil/sec.
+
+Wrapping this subroutine C<< sub { [200, "OK"] } >> and this simple metadata C<<
+{v=>1.1} >> using default options yields call performance for C<< $sub->() >> of
+about 0.9 mil/sec. With C<< validate_args=>0 >> and C<< validate_result=>0 >>,
+it's 1.5 mil/sec.
+
+As more (and more complex) arguments are introduced and validated, overhead will
+increase. The significant portion of the overhead is in argument validation. For
+example, this metadata C<< {v=>1.1, args=>{a=>{schema=>"int"}}} >> yields 0.5
+mil/sec.
+
 =head1 FUNCTIONS
 
 
@@ -1565,6 +1633,8 @@ wrapping code to source code during build.
 
 Wrap subroutine to do various things, like enforcing Rinci properties.
 
+This function is not exported by default, but exportable.
+
 Arguments ('*' denotes required arguments):
 
 =over 4
@@ -1677,97 +1747,10 @@ future nested wrapper can choose to avoid duplicating the same functionality.
 
 =for Pod::Coverage ^(new|handle(meta)?_.+|wrap|add_.+|section_empty|indent|unindent|get_indent_level|select_section|push_lines)$
 
-=head1 EXTENDING
-
-The framework is simple and extensible. Please delve directly into the source
-code for now. Some notes:
-
-The internal uses OO.
-
-The main wrapper building mechanism is in the C<wrap()> method.
-
-For each Rinci property, it will call C<handle_NAME()> wrapper handler method.
-The C<handlemeta_NAME()> methods are called first, to determine order of
-processing. You can supply these methods either by subclassing the class or,
-more simply, monkeypatching the method in the C<Perinci::Sub::Wrapper> package.
-
-The wrapper handler method will be called with a hash argument, containing these
-keys: B<value> (property value), B<new> (this key will exist if C<convert>
-argument of C<wrap()> exists, to convert a property to a new value).
-
-For properties that have name in the form of C<NAME1.NAME2.NAME3> (i.e., dotted)
-only the first part of the name will be used (i.e., C<handle_NAME1()>).
-
-=head1 VARIABLES
-
-=head2 $Log_Wrapper_Code (BOOL)
-
-Whether to log wrapper result. Default is from environment variable
-LOG_PERINCI_WRAPPER_CODE, or false. Logging is done with L<Log::Any> at trace
-level.
-
 =head1 METHODS
 
 The OO interface is only used internally or when you want to extend the wrapper.
 
-=head1 ENVIRONMENT
-
-=head2 LOG_PERINCI_WRAPPER_CODE (bool)
-
-If set to 1, will log the generated wrapper code. This value is used to set
-C<$Log_Wrapper_Code> if it is not already set.
-
-=head2 PERINCI_WRAPPER_CORE => bool
-
-Set default for wrap argument C<core>.
-
-=head2 PERINCI_WRAPPER_CORE_OR_PP => bool
-
-Set default for wrap argument C<core_or_pp>.
-
-=head2 PERINCI_WRAPPER_PP => bool
-
-Set default for wrap argument C<pp>.
-
-=head1 RINCI FUNCTION METADATA
-
-=head2 x.perinci.sub.wrapper.disable_validate_args => bool
-
-Can be set to 1 to set C<validate_args> to 0 by default. This is used e.g. if
-you already embed/insert code to validate arguments by other means and do not
-want to repeat validating arguments. E.g. used if you use
-L<Dist::Zilla::Plugin::Rinci::Validate>.
-
-=head2 x.perinci.sub.wrapper.disable_validate_result => bool
-
-Can be set to 1 to set C<validate_result> to 0 by default. This is used e.g. if
-you already embed/insert code to validate result by other means and do not want
-to repeat validating result. E.g. used if you use
-L<Dist::Zilla::Plugin::Rinci::Validate>.
-
-=head2 x.perinci.sub.wrapper.logs => array
-
-Generated/added by this module to the function metadata for every wrapping done.
-Used to avoid adding repeated code, e.g. to validate result or arguments.
-
-=head1 PERFORMANCE NOTES
-
-The following numbers are produced on an Intel Core i5-2400 3.1GHz desktop using
-PSW v0.51 and Perl v5.18.2. Operating system is Debian sid (64bit).
-
-For perspective, empty subroutine (C<< sub {} >>) as well as C<< sub { [200,
-"OK"] } >> can be called around 5.3 mil/sec.
-
-Wrapping this subroutine C<< sub { [200, "OK"] } >> and this simple metadata C<<
-{v=>1.1} >> using default options yields call performance for C<< $sub->() >> of
-about 0.9 mil/sec. With C<< validate_args=>0 >> and C<< validate_result=>0 >>,
-it's 1.5 mil/sec.
-
-As more (and more complex) arguments are introduced and validated, overhead will
-increase. The significant portion of the overhead is in argument validation. For
-example, this metadata C<< {v=>1.1, args=>{a=>{schema=>"int"}}} >> yields 0.5
-mil/sec.
-
 =head1 FAQ
 
 =head2 General
@@ -1842,15 +1825,24 @@ The solution is for your function to use the caller() replacement, provided by
 L<Perinci::Sub::Util>. Or use embedded mode, where the wrapper code won't add
 extra subroutine calls.
 
-=head1 SEE ALSO
+=head1 ENVIRONMENT
 
-L<Perinci>, L<Rinci>
+=head2 LOG_PERINCI_WRAPPER_CODE (bool)
 
-L<Python::Decorator>
+If set to 1, will log the generated wrapper code. This value is used to set
+C<$Log_Wrapper_Code> if it is not already set.
 
-L<Dist::Zilla::Plugin::Rinci::Wrap>
+=head2 PERINCI_WRAPPER_CORE => bool
 
-L<Dist::Zilla::Plugin::Rinci::Validate>
+Set default for wrap argument C<core>.
+
+=head2 PERINCI_WRAPPER_CORE_OR_PP => bool
+
+Set default for wrap argument C<core_or_pp>.
+
+=head2 PERINCI_WRAPPER_PP => bool
+
+Set default for wrap argument C<pp>.
 
 =head1 HOMEPAGE
 
@@ -1868,13 +1860,23 @@ When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
 feature.
 
+=head1 SEE ALSO
+
+L<Perinci>, L<Rinci>
+
+L<Python::Decorator>
+
+L<Dist::Zilla::Plugin::Rinci::Wrap>
+
+L<Dist::Zilla::Plugin::Rinci::Validate>
+
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2016 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
